package com.emall.flash.mobile.controller;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.shop.ActivityInfo;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.shop.ActivityService;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.DateUtil;
import com.emall.flash.utils.factory.Page;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.*;

import java.text.SimpleDateFormat;
import java.util.Calendar;


@RestController
@RequestMapping("/activity")
public class ActivityController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private ActivityService activityService;

    /**
     * 获取所有活动
     */
    @RequestMapping(value = "/list",method = RequestMethod.GET)
    public Object list(@RequestParam(required = false) Long tenantId,
                       @RequestParam(required = false) Integer orderBy,
                       @RequestParam(required = false) String name) {
        Page page = new PageFactory().defaultPage();
        if (tenantId!=null)
            page.addFilter(SearchFilter.build("tenantId", tenantId));
        if (StringUtils.isNotBlank(name))
            page.addFilter(SearchFilter.build("name", SearchFilter.Operator.LIKE, name));
        page.addFilter(SearchFilter.build("delFlag", 0));
        page.addFilter(SearchFilter.build("status", SearchFilter.Operator.EQ,1));
        if (orderBy!=null) {
            switch (orderBy){ //排序方式 0:全部 1：最新 2：近一个月
                case 0:
                    page.setSort(Sort.by(Sort.Direction.DESC, "createTime"));
                    break;
                case 1:
                    page.setSort(Sort.by(Sort.Direction.DESC, "activityBeginDate"));
                    break;
                case 2:
                    Calendar date = Calendar.getInstance();
                    SimpleDateFormat dd  = new SimpleDateFormat("yyyyMMddHHmmss");
                    date.add(Calendar.DATE, -30);
                    String formatTime = dd.format(date.getTime());
                    page.addFilter("activityBeginDate", SearchFilter.Operator.GTE, DateUtil.parse(formatTime,"yyyyMMddHHmmss"));
                    page.setSort(Sort.by(Sort.Direction.DESC, "activityBeginDate"));
                    break;
                default:
                    page.setSort(Sort.by(Sort.Direction.DESC, "createTime"));
                    break;
            }
        }
        page = activityService.queryPage(page);
        return Rets.success(page);
    }


    /**
     * 获取所有活动
     */
    @RequestMapping(value = "/get",method = RequestMethod.GET)
    public Object get(@RequestParam(required = false) Long id) {
        ActivityInfo activityInfo = activityService.get(id);
        return Rets.success(activityInfo);
    }


}
