package com.emall.flash.mobile.controller;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.cache.CacheDao;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.shop.*;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.Maps;
import com.emall.flash.utils.RedisCons;
import com.emall.flash.utils.StringUtil;
import com.emall.flash.utils.factory.Page;
import com.emall.flash.web.controller.BaseController;
import com.emall.flash.bean.entity.shop.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ：enilu
 * @date ：Created in 11/5/2019 11:16 AM
 */
@RestController
@RequestMapping("/goods")
public class GoodsController extends BaseController {
    @Autowired
    private GoodsService goodsService;
    @Autowired
    private GoodsSkuService goodsSkuService;
    @Autowired
    private CategoryService categoryService;

    @Autowired
    private AttrKeyService attrKeyService;
    @Autowired
    private FavoriteService favoriteService;
    @Autowired
    private ColumnsGoodsService columnsGoodsService;
    @Autowired
    private CacheDao cacheDao;

    /**
     * 获取指定类别下的商品列表
     *
     * @param idCategory
     * @return
     */
    @RequestMapping(value = "/queryGoods", method = RequestMethod.GET)
    public Object queryGoods(@RequestParam("idCategory") Long idCategory,@RequestParam("tenantId") Long tenantId) {
        Page<Goods> page = new PageFactory<Goods>().defaultPage();
        List<Category> categories = categoryService.queryAll(SearchFilter.build("pid", idCategory));
        List<Long> ids = Lists.newArrayList(idCategory);
        categories.forEach(item -> {
            ids.add(item.getId());
        });
        if (ids.size() == 1) {
            page.addFilter(SearchFilter.build("idCategory", SearchFilter.Operator.EQ, idCategory));
        } else {
            page.addFilter(SearchFilter.build("idCategory", SearchFilter.Operator.IN, ids));
        }
        page.addFilter(SearchFilter.build("tenantId", tenantId));
        page.addFilter(SearchFilter.build("isOnSale", true));
        page = goodsService.queryPage(page);
        return Rets.success(page);
    }

    /**
     * 获取指定栏目下的商品列表
     *
     * @param columnsId
     * @return
     */
    @RequestMapping(value = "/queryGoodsByColumnsId", method = RequestMethod.GET)
    public Object queryGoodsByColumnsId(@RequestParam("columnsId") Long columnsId,@RequestParam("tenantId") Long tenantId,
                                        @RequestParam(value = "name",required = false) String name) {
        List<SearchFilter> filters = Lists.newArrayList();
        filters.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ,0));
        filters.add(SearchFilter.build("columnsId",columnsId));
        List<ColumnsGoods> list = columnsGoodsService.queryAll(filters);
        Page<Goods> page = new PageFactory<Goods>().defaultPage();
        if (!list.isEmpty()) {
            List<Long> goodsIds = list.stream().map(ColumnsGoods::getGoodsId).collect(Collectors.toList());
            page.addFilter(SearchFilter.build("id", SearchFilter.Operator.IN, goodsIds));
            if (StringUtils.isNotBlank(name))
                page.addFilter(SearchFilter.build("name", SearchFilter.Operator.LIKE, name));
            page.addFilter(SearchFilter.build("tenantId", tenantId));
            page.addFilter(SearchFilter.build("isOnSale", true));
            page.addFilter(SearchFilter.build("isDelete", false));
            page = goodsService.queryPage(page);
            return Rets.success(page);
        } else
            return Rets.success(page);
    }

    /**
     * 根据关键字搜索商品
     *
     * @param key
     * @return
     */
    @RequestMapping(value = "/search", method = RequestMethod.GET)
    public Object search(@RequestParam("key") String key,@RequestParam("tenantId") Long tenantId) {
        Page<Goods> page = new PageFactory<Goods>().defaultPage();
        if (StringUtil.isNotEmpty(key)) {
            page.addFilter(SearchFilter.build("name", SearchFilter.Operator.LIKE, key));
        }
        if (tenantId!=null)
            page.addFilter(SearchFilter.build("tenantId", SearchFilter.Operator.EQ, tenantId));
        page.addFilter(SearchFilter.build("isOnSale", true));
        page = goodsService.queryPage(page);
        return Rets.success(page);
    }

    @RequestMapping(value = "/searchNew", method = RequestMethod.GET)
    public Object searchNew(@RequestParam("tenantId") Long tenantId) {
        List<Goods> list = goodsService.queryAll(Lists.newArrayList(
                SearchFilter.build("isNew", true),
                SearchFilter.build("isOnSale", true),
                SearchFilter.build("tenantId", tenantId)
        ));
        return Rets.success(list);
    }

    /**
     * 查询热门商品
     *
     * @return
     */
    @RequestMapping(value = "/searchHot", method = RequestMethod.GET)
    public Object searchHot(@RequestParam("tenantId") Long tenantId) {
        List<Goods> list = goodsService.queryAll(Lists.newArrayList(
                SearchFilter.build("isHot", true),
                SearchFilter.build("isOnSale", true),
                SearchFilter.build("tenantId", tenantId)
        ));
        return Rets.success(list);
    }

    @RequestMapping(value = "/{id}", method = RequestMethod.GET)
    public Object get(@PathVariable Long id) {
        Goods goods = goodsService.get(id);
        List<GoodsSku> skuList = goodsSkuService.queryAll(Lists.newArrayList(
                SearchFilter.build("idGoods", id),
                SearchFilter.build("isDeleted", false)
        ));


        Map skuMap = Maps.newHashMap();

        List<Map> tree = Lists.newArrayList();

        if (!skuList.isEmpty()) {
            List<AttrVal> attrValList = Lists.newArrayList();
            List<AttrKey> attrKeyList = attrKeyService.queryBy(goods.getIdCategory());
            for (AttrKey attrKey : attrKeyList) {
                Map treeNode = Maps.newHashMap();
                treeNode.put("k", attrKey.getAttrName());
                List<Map> v = Lists.newArrayList();
                List<AttrVal> attrValListChildren = attrKey.getAttrVals();
                attrValList.addAll(attrValListChildren);
                for (AttrVal attrVal : attrValListChildren) {
                    v.add(Maps.newHashMap(
                            "id", attrVal.getId(),
                            "name", attrVal.getAttrVal(),
                            "plain", true
                    ));
                }
                treeNode.put("v", v);
                treeNode.put("k_s", "s" + attrKey.getId());
                tree.add(treeNode);
            }
            Map<Long, AttrVal> attrValMap = Lists.toMap(attrValList, "id");
            List<Map> skuList2 = Lists.newArrayList();

            for (GoodsSku sku : skuList) {
                Map oneSkuMap = Maps.newHashMap();
                oneSkuMap.put("id", sku.getId());
                oneSkuMap.put("price", sku.getPrice());
                String[] attrValIdArr = sku.getCode().split(",");
                for (String attrValId : attrValIdArr) {
                    AttrVal attrVal = attrValMap.get(Long.valueOf(attrValId));
                    oneSkuMap.put("s" + attrVal.getIdAttrKey(), attrVal.getId());
                }
                Integer count = cacheDao.hget(RedisCons.GOODS_SKU_COUNT ,sku.getId(),Integer.class);
                oneSkuMap.put("stock_num", count);
                oneSkuMap.put("code", sku.getCode());
                skuList2.add(oneSkuMap);
            }
            skuMap.put("list", skuList2);
            skuMap.put("price", skuList.get(0).getPrice());
            skuMap.put("collection_id", skuList.get(0).getId());
            skuMap.put("none_sku", false);
        } else {
            skuMap.put("price", goods.getPrice());
            skuMap.put("collection_id", goods.getId());
            skuMap.put("none_sku", true);
        }
        skuMap.put("tree", tree);
        Integer count = cacheDao.hget(RedisCons.GOODS_COUNT ,id,Integer.class);
        skuMap.put("stock_num", count);
        skuMap.put("hide_stock", false);
        goods.setStock(count);
        Map result = Maps.newHashMap(
                "goods", goods,
                "sku", skuMap
        );

        //如果用户已登录，则获取用户是否收藏该商品
        Long idUser = JwtUtil.getUserId();
        if (idUser != null) {
            Favorite favorite = favoriteService.get(idUser, id);
            result.put("favorite", favorite != null);
        } else {
            result.put("favorite", false);
        }

        return Rets.success(result);
    }
}
