package com.emall.flash.mobile.controller;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.enumeration.shop.OrderEnum;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.service.WeixinPayService;
import com.emall.flash.service.api.express.kdniao.KdniaoService;
import com.emall.flash.service.shop.*;
import com.emall.flash.service.system.CfgService;
import com.emall.flash.utils.HttpUtil;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.Maps;
import com.emall.flash.utils.StringUtil;
import com.emall.flash.utils.factory.Page;
import com.emall.flash.web.controller.BaseController;
import com.emall.flash.bean.entity.shop.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author ：enilu
 * @date ：Created in 11/6/2019 5:07 PM
 */
@RestController
@RequestMapping("/user/order")
public class OrderController extends BaseController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private OrderService orderService;
    @Autowired
    private CartService cartService;
    @Autowired
    private AddressService addressService;
    @Autowired
    private GoodsService goodsService;
    @Autowired
    private GoodsSkuService goodsSkuService;
    @Autowired
    private CfgService cfgService;
    @Autowired
    private KdniaoService kdniaoService;
    @Autowired
    private WeixinPayService weixinPayService;

    @RequestMapping(value = "{orderSn}", method = RequestMethod.GET)
    public Object get(@PathVariable(value = "orderSn") String orderSn) {
        Order order = orderService.getByOrderSn(orderSn);
        return Rets.success(order);
    }

    @RequestMapping(value = "getOrders", method = RequestMethod.GET)
    public Object getOrders(@RequestParam(value = "status", required = false) Integer status,
                            @RequestParam(value = "tenantId", required = false) Long tenantId) {
        Long idUser = getIdUser(HttpUtil.getRequest());
        Page<Order> page = new PageFactory<Order>().defaultPage();
        page.addFilter(SearchFilter.build("idUser", SearchFilter.Operator.EQ, idUser));
        page.setSort(Sort.by(Sort.Direction.DESC, "id"));
        if (status != null && status != 0) {
            if (status == 5)
                page.addFilter(SearchFilter.build("status", SearchFilter.Operator.IN, Lists.newArrayList(6,7,8)));
            else
                page.addFilter(SearchFilter.build("status", SearchFilter.Operator.EQ, status));
        }
        if (tenantId != null) {
            page.addFilter(SearchFilter.build("tenantId", SearchFilter.Operator.EQ, tenantId));
        }
        page = orderService.queryPage(page);
        return Rets.success(page);
    }

    @RequestMapping(value = "prepareCheckout", method = RequestMethod.GET)
    public Object prepareCheckout(@RequestParam(value = "chosenAddressId", required = false) Long chosenAddressId,
                                  @RequestParam(value = "idCarts") String idCarts) {
        Long idUser = getIdUser(HttpUtil.getRequest());
        List<SearchFilter> filters = Lists.newArrayList(
                SearchFilter.build("idUser", SearchFilter.Operator.EQ, idUser),
                SearchFilter.build("id", SearchFilter.Operator.IN, StringUtil.splitForLong(idCarts, ",")),
                SearchFilter.build("delFlag", 0)
        );
        List<Cart> list = cartService.queryAll(filters);
        Address address = null;
        if (chosenAddressId == null || chosenAddressId == 0) {
            address = addressService.getDefaultAddr(idUser);
        } else {
            address = addressService.get(chosenAddressId);
            if(address==null){
                address = addressService.getDefaultAddr(idUser);
            }
        }
        return Rets.success(Maps.newHashMap(
                "list", list, "addr", address
        ));
    }

    @RequestMapping(value = "prepareCheckoutAddress", method = RequestMethod.GET)
    public Object prepareCheckoutAddress(@RequestParam(value = "chosenAddressId", required = false) Long chosenAddressId) {
        Long idUser = getIdUser(HttpUtil.getRequest());

        Address address = null;
        if (chosenAddressId == null || chosenAddressId == 0) {
            address = addressService.getDefaultAddr(idUser);
        } else {
            address = addressService.get(chosenAddressId);
            if(address==null){
                address = addressService.getDefaultAddr(idUser);
            }
        }
        return Rets.success(Maps.newHashMap("addr", address
        ));
    }

//    @RequestMapping(value = "save", method = RequestMethod.POST)
//    @Transactional
//    public Object save(
//            @RequestParam("idAddress") Long idAddress,
//            @RequestParam(value = "message", required = false) String message,
//            @RequestParam(value = "idCarts") String idCarts,
//            @RequestParam(value = "tenantId", required = false) Long tenantId
//    ) {
//
//        Long idUser = getIdUser();
//        List<SearchFilter> filters = Lists.newArrayList(
//                SearchFilter.build("idUser", SearchFilter.Operator.EQ, idUser),
//                SearchFilter.build("id", SearchFilter.Operator.IN, StringUtil.splitForLong(idCarts, ",")),
//                SearchFilter.build("delFlag", 0)
//        );
//        List<Cart> cartList = cartService.queryAll(filters);
//
//        Order order = new Order();
//        order.setIdUser(idUser);
//        Address address = addressService.get(idAddress);
//        order.setConsignee(address.getName());
//        order.setConsigneeAddress(address.getWholeAddressInfo());
//        order.setMobile(address.getTel());
//        BigDecimal totalPrice = new BigDecimal(0);
//        List<OrderItem> itemList = Lists.newArrayList();
//        for (Cart cart : cartList) {
//            OrderItem orderItem = new OrderItem();
//            orderItem.setIdGoods(cart.getIdGoods());
//            orderItem.setIdSku(cart.getIdSku());
//            orderItem.setPrice(cart.getPrice());
//            orderItem.setCount(cart.getCount());
//            orderItem.setTotalPrice(orderItem.getPrice().multiply(new BigDecimal(orderItem.getCount())));
//            orderItem.setTenantId(tenantId);
//            totalPrice = totalPrice.add(orderItem.getTotalPrice());
//            itemList.add(orderItem);
//        }
//        order.setMessage(message);
//        order.setTotalPrice(totalPrice);
//        order.setRealPrice(totalPrice);
//        order.setStatus(OrderEnum.OrderStatusEnum.UN_PAY.getId());
//        order.setPayStatus(OrderEnum.PayStatusEnum.UN_PAY.getId());
//        order.setTenantId(tenantId);
//        orderService.save(order, itemList);
//        cartService.deleteAll(cartList);
//        return Rets.success(order);
//    }

    @RequestMapping(value = "saveOrder", method = RequestMethod.POST)
    @Transactional
    public Object saveOrder(
            @RequestParam("idAddress") Long idAddress,
            @RequestParam(value = "message", required = false) String message,
            @RequestParam(value = "idGoods") Long idGoods,
            @RequestParam(value = "idSku") Long idSku,
            @RequestParam(value = "count") Integer count,
            @RequestParam(value = "tenantId", required = false) Long tenantId
    ) {
        Long idUser = getIdUser();

        Order order = new Order();
        order.setIdUser(idUser);
        Address address = addressService.get(idAddress);
        if (address!=null) {
            order.setConsignee(address.getName());
            order.setConsigneeAddress(address.getWholeAddressInfo());
            order.setMobile(address.getTel());
        }
        BigDecimal totalPrice = new BigDecimal(0);
        List<OrderItem> itemList = Lists.newArrayList();
//        for (Cart cart : cartList) {
            OrderItem orderItem = new OrderItem();
            orderItem.setIdGoods(idGoods);
            orderItem.setIdSku(idSku);
            Goods goods = goodsService.get(idGoods);
            if (idSku!=null) {
                GoodsSku goodsSku = goodsSkuService.get(idSku);
                orderItem.setPrice(goodsSku.getPrice());
            } else
                orderItem.setPrice(goods.getPrice());
            orderItem.setCount(count);
            orderItem.setTotalPrice(orderItem.getPrice().multiply(new BigDecimal(orderItem.getCount())));
            orderItem.setTenantId(tenantId);
            totalPrice = totalPrice.add(orderItem.getTotalPrice());
            itemList.add(orderItem);
//        }
        order.setMessage(message);
        order.setTotalPrice(totalPrice);
        order.setRealPrice(totalPrice);
        order.setStatus(OrderEnum.OrderStatusEnum.UN_PAY.getId());
        order.setPayStatus(OrderEnum.PayStatusEnum.UN_PAY.getId());
        order.setTenantId(tenantId);
        orderService.save(order, itemList,idGoods);

        return Rets.success(order);
    }

    @RequestMapping(value = "cancel/{orderSn}", method = RequestMethod.POST)
    public Object cancel(@PathVariable("orderSn") String orderSn) {
        orderService.cancel(orderSn);
        return Rets.success();
    }

    @RequestMapping(value = "confirm/{orderSn}", method = RequestMethod.POST)
    public Object confirm(@PathVariable("orderSn") String orderSn) {
        Order order = orderService.confirmReceive(orderSn);
        return Rets.success(order);
    }

    @RequestMapping(value = "getExpressInfo/{orderSn}", method = RequestMethod.GET)
    public Object getExpressInfo(@PathVariable(value = "orderSn") String orderSn) {
        Order order = orderService.getByOrderSn(orderSn);
        ExpressInfo response = orderService.getExpressInfo(order.getOrderSn());
        Map data = Maps.newHashMap(
                "order", order,
                "expressInfo", response
        );
        return Rets.success(data);
    }

    /**
     * 用户订单退款
     *
     * @param orderSn
     */
    @RequestMapping(value = "refund/{orderSn}", method = RequestMethod.POST)
    public Object refund(@PathVariable("orderSn") String orderSn,
                         @RequestParam(required = false)  String refundSn) {
        Order order = orderService.getByOrderSn(orderSn);
        //如果订单状态是待发货，可以直接退款
        if (order.getStatus().equals(OrderEnum.OrderStatusEnum.UN_SEND.getId())){
            weixinPayService.refund(order);
        } else {
            order.setRefundSn(refundSn);
            order.setStatus(OrderEnum.OrderStatusEnum.UN_REFUND.getId());
            orderService.updateOrder(order);
        }
        String descript = "用户申请退款";
        orderService.saveOrderLog(order, descript);
        return Rets.success();
    }
}
