package com.emall.flash.mobile.controller;

import com.emall.flash.bean.entity.shop.Tenant;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.service.shop.TenantService;
import com.emall.flash.service.system.CityService;
import com.emall.flash.utils.GeoUtil;
import com.emall.flash.utils.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.*;

import java.util.Comparator;
import java.util.List;


@RestController
@RequestMapping("/shop/tenant/mobile")
public class TenantController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private TenantService tenantService;
    @Autowired
    private CityService cityService;

    /**
     * 获取所有商圈
     */
    @RequestMapping(value = "/list",method = RequestMethod.GET)
//    @RequiresPermissions(value = {Permission.DICT})
    public Object list(@RequestParam(required = false) Long cityId,
                       @RequestParam(required = false) String latitude,
                       @RequestParam(required = false) String name) {
        List<SearchFilter> filters = Lists.newArrayList();
        if (cityId!=null)
            filters.add(SearchFilter.build("cityId", SearchFilter.Operator.EQ, cityId));
        filters.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ, 0));
        if (StringUtils.isNotBlank(name))
            filters.add(SearchFilter.build("name", SearchFilter.Operator.LIKE, name));
        List<Tenant> list = tenantService.queryAll(filters, Sort.by(Sort.Direction.ASC, "sort"));
        //经纬度排序
        if (StringUtils.isNotBlank(latitude)){
            String[] latitudes = latitude.split(",");
            double longitudeOne = Double.parseDouble(latitudes[0]);
            double latitudeOne = Double.parseDouble(latitudes[1]);
            list.forEach(l -> {
                double longitudeTwo = Double.parseDouble(l.getLatitude().split(",")[0]);
                double latitudeTwo = Double.parseDouble(l.getLatitude().split(",")[1]);
                l.setDistance(GeoUtil.getDistance(longitudeOne, latitudeOne,
                        longitudeTwo,latitudeTwo));
            });
            list.sort(Comparator.comparing(Tenant::getDistance));
        }
        list.forEach(l -> l.setCityName(cityService.get(l.getCityId()).getName()));
        return Rets.success(list);
    }


}
