package com.emall.flash.mobile.controller;


import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.constant.state.IntegralStyle;
import com.emall.flash.bean.entity.shop.TenantUser;
import com.emall.flash.bean.entity.shop.UserIntegral;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.shop.TenantUserService;
import com.emall.flash.service.shop.UserIntegralService;
import com.emall.flash.utils.GeoUtil;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.factory.Page;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Comparator;
import java.util.List;


@RestController
@RequestMapping("/user/tenantUser/mobile")
public class TenantUserController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private TenantUserService tenantUserService;
    @Autowired
    private UserIntegralService userIntegralService;
    /**
     * 获取对应商圈的会员等级积分
     */
    @RequestMapping(value = "/get",method = RequestMethod.GET)
    public Object get(@RequestParam(required = false) Long tenantId) {
        List<SearchFilter> filters = Lists.newArrayList();
        if (tenantId!=null)
            filters.add(SearchFilter.build("tenantId", SearchFilter.Operator.EQ, tenantId));
        Long idUser = JwtUtil.getUserId();
        filters.add(SearchFilter.build("userId", SearchFilter.Operator.EQ, idUser));
        filters.add(SearchFilter.build("delFlag", SearchFilter.Operator.EQ, 0));
        TenantUser tenantUser = tenantUserService.get(filters);
        return Rets.success(tenantUser);
    }

    /**
     * 获取对应商圈的用户积分明细
     */
    @RequestMapping(value = "/userIntegralList",method = RequestMethod.GET)
    public Object userIntegralList(@RequestParam(required = false) Long tenantId) {
        Page<UserIntegral> page = new PageFactory().defaultPage();
        if (tenantId!=null)
            page.addFilter(SearchFilter.build("tenantId", tenantId));
        Long idUser = JwtUtil.getUserId();
        page.addFilter(SearchFilter.build("userId", SearchFilter.Operator.EQ, idUser));
        page.addFilter(SearchFilter.build("delFlag", 0));
        page.setSort(Sort.by(Sort.Direction.DESC, "createTime"));
        page = userIntegralService.queryPage(page);
        for (UserIntegral record : page.getRecords()) {
            record.setStyleName(IntegralStyle.valueOf(record.getStyle()));
        }
        return Rets.success(page);
    }


}
