package com.emall.flash.mobile.controller;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.entity.shop.*;
import com.emall.flash.bean.enumeration.shop.OrderEnum;
import com.emall.flash.bean.exception.ApplicationException;
import com.emall.flash.bean.exception.ApplicationExceptionEnum;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.cache.CacheDao;
import com.emall.flash.service.shop.ActivityUserService;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.RedisCons;
import com.emall.flash.utils.StringUtil;
import com.emall.flash.utils.factory.Page;
import com.emall.flash.web.controller.BaseController;
import org.apache.commons.collections.CollectionUtils;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;

@RestController
@RequestMapping("/user/activity")
public class UserActivityController extends BaseController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private ActivityUserService activityUserService;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private CacheDao cacheDao;
    @Resource
    private RedisTemplate<String, Object> redisTemplate;


    /**
     * 获取我的活动
     */
    @RequestMapping(value = "/list",method = RequestMethod.GET)
    public Object list(@RequestParam(required = false) Long tenantId,
                       @RequestParam(required = false) Integer status) {
        Page page = new PageFactory().defaultPage();
        page.setSort(Sort.by(Sort.Direction.DESC, "create_time"));
        page = activityUserService.selectPage(page,status,tenantId);
        return Rets.success(page);
    }

    //活動下單
    @RequestMapping(value = "save", method = RequestMethod.POST)
    public Object save(@RequestParam("tenantId") Long tenantId,
                       @RequestParam("activityId") Long activityId,
                       @RequestParam("count") Integer count) {
        if (count > 5 )
            throw new ApplicationException(ApplicationExceptionEnum.ACTIVITY_COUNT_ERROR);
        //加redis锁
        boolean isGetLock = false;
        RLock lock = redissonClient.getLock(RedisCons.ACTIVITY_INFO + activityId);
        try {
            isGetLock = lock.tryLock(Long.parseLong(RedisCons.ORDER_INFO_LOCK_WAIT_TIME),
                    Long.parseLong(RedisCons.ORDER_INFO_LOCK_LEASE_TIME), TimeUnit.SECONDS);
            if (isGetLock) {
                //获取redis【activity:detail:${aid}】
                ActivityInfo detail = cacheDao.hget(RedisCons.ACTIVITY_DETAIL ,activityId,ActivityInfo.class);
                if (detail!=null) {
                    if (detail.getActivityEndDate().before(new Date()))
                        throw new ApplicationException(ApplicationExceptionEnum.ACTIVITY_END_ERROR);
                    if (detail.getActivityBeginDate().before(new Date()))
                        throw new ApplicationException(ApplicationExceptionEnum.ACTIVITY_ING_ERROR);
                    Integer num = cacheDao.hget(RedisCons.ACTIVITY_COUNT , activityId,Integer.class);
                    //如果强制人数
                    if (detail.getIsRestrictions().equals(0) && ((num-count)<0))
                        throw new ApplicationException(ApplicationExceptionEnum.ACTIVITY_ORDER_ERROR);
                    Long idUser = getIdUser();
                    ActivityUser activityUser = new ActivityUser();
                    activityUser.setActivityId(activityId);
                    activityUser.setTenantId(tenantId);
                    activityUser.setUserId(idUser);
                    activityUser.setCount(count);
                    activityUser.setStatus(0);
                    activityUserService.insert(activityUser);
                    //扣庫存
                    redisTemplate.opsForValue().increment(RedisCons.ACTIVITY_COUNT+":"+activityId, -count);
                    cacheDao.hset(RedisCons.ORDER_ACTIVITY , activityId,activityUser);
                    return Rets.success(activityUser);
                }
            }
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            isGetLock = false;
        } finally {
            lock.unlock();
            logger.error("释放成功");
        }
        return Rets.failure("下单失败");
    }
}
