package com.emall.flash.mobile.controller;

import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.core.BussinessLog;
import com.emall.flash.bean.entity.shop.Parking;
import com.emall.flash.bean.entity.shop.ParkingPayment;
import com.emall.flash.bean.entity.shop.Receipt;
import com.emall.flash.bean.entity.shop.UserCar;
import com.emall.flash.bean.entity.system.User;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.shop.ParkingPaymentService;
import com.emall.flash.service.shop.ParkingService;
import com.emall.flash.service.shop.UserCarService;
import com.emall.flash.service.system.ManagerService;
import com.emall.flash.utils.Lists;
import com.emall.flash.utils.factory.Page;
import com.emall.flash.web.controller.BaseController;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;


@RestController
@RequestMapping("/user/car")
public class UserCarController extends BaseController {
    private Logger logger = LoggerFactory.getLogger(getClass());
    @Autowired
    private UserCarService userCarService;
    @Autowired
    private ParkingPaymentService parkingPaymentService;

    /**
     * 获取用户车辆
     */
    @RequestMapping(value = "/list",method = RequestMethod.GET)
    public Object list() {
        List<SearchFilter> filters = Lists.newArrayList();
        Long userId = getIdUser();
        filters.add(SearchFilter.build("delFlag", 0));
        filters.add(SearchFilter.build("userId",userId));
        Sort sort = Sort.by(Sort.Direction.DESC, "createTime");
        List<UserCar> list = userCarService.queryAll(filters,sort);
        return Rets.success(list);
    }

    @RequestMapping(value = "/add",method = RequestMethod.POST)
    @BussinessLog(value = "添加车辆", key = "id")
    public Object add(@RequestBody UserCar userCar) {
        userCar.setDelFlag(0);
        Long userId = getIdUser();
        userCar.setUserId(userId);
        if (userCarService.isRepeat(userCar))
            return Rets.failure("该车辆已经存在");
        userCarService.insert(userCar);
        return Rets.success(userCar);
    }
//
//    @RequestMapping(value = "/update",method = RequestMethod.PUT)
//    @BussinessLog(value = "修改停车场", key = "id")
//    public Object update(@RequestBody Parking parking) {
//        if (parkingService.isRepeat(parking))
//            return Rets.failure("停车场名称重复");
//        parkingService.update(parking);
//        return Rets.success();
//    }


    @RequestMapping(method = RequestMethod.DELETE)
    @BussinessLog(value = "删除车辆", key = "id")
    public Object delete(@RequestParam Long id) {
        UserCar userCar = userCarService.get(id);
        userCar.setDelFlag(1);
        userCarService.update(userCar);
        return Rets.success();
    }

    /**
     * 查询停车记录
     */
    @RequestMapping(value = "/parkingPaymentList",method = RequestMethod.GET)
    public Object parkingPaymentList(@RequestParam(required = false) Long tenantId,
                                     @RequestParam(required = false) Long parkingId) {
        Page<ParkingPayment> page = new PageFactory().defaultPage();
        if (tenantId!=null)
            page.addFilter(SearchFilter.build("tenantId", tenantId));
        if (parkingId!=null)
            page.addFilter(SearchFilter.build("parkingId", parkingId));
        Long idUser = getIdUser();
        page.addFilter(SearchFilter.build("userId", idUser));
        page.addFilter(SearchFilter.build("delFlag", 0));
        page.setSort(Sort.by(Sort.Direction.DESC, "createTime"));
        page = parkingPaymentService.queryPage(page);
        return Rets.success(page);
    }
    /**根据查询的缴费记录向停车缴费表插入数据**/
    @RequestMapping(value = "/inquiryPayment",method = RequestMethod.GET)
    public Object inquiryPayment(@RequestParam(required = false) Long tenantId,
                                 @RequestParam(required = false) Long parkingId,
                                 @RequestParam(required = false) Long userId,
                                 @RequestParam(required = false) String car){
        // 需要查第三方接口,如果查到了向数据库中停车缴费记录表插入一条数据,如果没有返回null
        // 造一条假的缴费记录
        ParkingPayment parkingPayment = new ParkingPayment();
        parkingPayment.setTenantId(tenantId);
        parkingPayment.setParkingId(parkingId);
        parkingPayment.setUserId(userId);
        parkingPayment.setCar(car);
        parkingPayment.setInTime(new Date());
        parkingPayment.setOutTime(new Date());
        parkingPayment.setAmount(BigDecimal.valueOf(200));
        parkingPayment.setPayStatus(0);
        parkingPayment.setDelFlag(0);
        parkingPaymentService.insert(parkingPayment);
        return Rets.success(parkingPayment);
    }
}
