package com.emall.flash.mobile.controller;

import cn.hutool.core.codec.Base64Encoder;
import com.emall.flash.bean.constant.factory.PageFactory;
import com.emall.flash.bean.entity.shop.ShopUser;
import com.emall.flash.bean.entity.system.FileInfo;
import com.emall.flash.bean.vo.UserInfo;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.query.SearchFilter;
import com.emall.flash.bean.vo.shop.WechatInfo;
import com.emall.flash.cache.CacheDao;
import com.emall.flash.dao.shop.ShopRepository;
import com.emall.flash.security.JwtUtil;
import com.emall.flash.service.api.WeixinService;
import com.emall.flash.service.shop.ActivityService;
import com.emall.flash.service.shop.ActivityUserService;
import com.emall.flash.service.shop.ShopService;
import com.emall.flash.service.shop.ShopUserService;
import com.emall.flash.service.system.FileService;
import com.emall.flash.utils.CyQrCodeUtil;
import com.emall.flash.utils.DateUtil;
import com.emall.flash.utils.MD5;
import com.emall.flash.utils.StringUtil;
import com.emall.flash.utils.factory.Page;
import com.emall.flash.web.controller.BaseController;
import com.google.protobuf.ServiceException;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Map;

/**
 * @author ：enilu
 * @date ：Created in 11/6/2019 4:20 PM
 */
@RestController
@RequestMapping("/user")
public class UserController extends BaseController {
    @Autowired
    private ShopService shopService;
    @Autowired
    private ShopUserService shopUserService;
    @Autowired
    private WeixinService weixinService;
    @Autowired
    private FileService fileService;
    @Autowired
    private CacheDao cacheDao;
    @Autowired
    private ActivityUserService activityUserService;

    @RequestMapping(value = "/getInfo",method = RequestMethod.GET)
    public Object getInfo(){
        String token = getToken();
        Long idUser = JwtUtil.getUserId(token);
        ShopUser shopUser = shopUserService.get(idUser);
        UserInfo userInfo = new UserInfo();
        BeanUtils.copyProperties(shopUser,userInfo);
        WechatInfo wechatInfo = cacheDao.hget(CacheDao.SESSION,"WECHAT_INFO"+shopUser.getId(),WechatInfo.class);
        if(wechatInfo!=null){
            userInfo.setRefreshWechatInfo(false);
        }
        return Rets.success(userInfo);
    }
    @RequestMapping(value = "/update",method = RequestMethod.POST)
    public Object update(@RequestBody ShopUser shopUser){
        if (shopUser.getBirthdayString() != null && !shopUser.getBirthdayString().equals("")) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            Date date = null;
            try {
                date = sdf.parse(shopUser.getBirthdayString());
            } catch (ParseException e) {
                e.printStackTrace();
            }
            shopUser.setBirthday(date);
        }
       shopService.updateShopUser(shopUser);
       return Rets.success(shopUser);
    }
    @RequestMapping(value = "/updateUserName/{userName}",method = RequestMethod.POST)
    public Object updateUserName(@PathVariable("userName") String userName){
        ShopUser user = shopUserService.getCurrentUser();
        user.setNickName(userName);
        shopUserService.update(user);
        return Rets.success(user);
    }

    @RequestMapping(value = "/updateGender/{gender}",method = RequestMethod.POST)
    public Object updateGender(@PathVariable("gender") String gender){
        ShopUser user = shopUserService.getCurrentUser();
        user.setGender(gender);
        shopUserService.update(user);
        return Rets.success(user);
    }

    @RequestMapping(value = "/updatePassword/{oldPwd}/{password}/{rePassword}",method = RequestMethod.POST)
    public Object updatePassword(@PathVariable("oldPwd") String oldPwd,
                                 @PathVariable("password") String password,
                                 @PathVariable("rePassword") String rePassword){
        if(StringUtil.isEmpty(oldPwd) || StringUtil.isEmpty(password) || StringUtil.isEmpty(rePassword)){
            return  Rets.failure("项目并能为空");
        }
        if(!StringUtil.equals(password,rePassword)){
            return Rets.failure("密码前后不一致");
        }
        ShopUser user = shopUserService.getCurrentUser();
        String oldPasswdMd5 = MD5.md5(oldPwd, user.getSalt());
        if(!StringUtil.equals(oldPasswdMd5,user.getPassword())){
            return Rets.failure("旧密码不正确");
        }
        user.setPassword(MD5.md5(password,user.getSalt()));
        shopUserService.update(user);
        return Rets.success();
    }
    @RequestMapping(value = "/updatePassword_v2/{password}/{smsCode}",method = RequestMethod.POST)
    public Object updatePassword(@PathVariable("password") String password,
                                 @PathVariable("smsCode") String smsCode){
        ShopUser user = shopUserService.getCurrentUser();
        Boolean validateRet = shopUserService.validateSmsCode(user.getMobile(), smsCode);
        if (validateRet) {
            user.setPassword(MD5.md5(password,user.getSalt()));
            shopUserService.update(user);
            return Rets.success();
        }
        return Rets.failure("短信验证码错误");

    }
    @RequestMapping(value = "sendSmsCode",method = RequestMethod.POST)
    public Object sendSmsCode(@RequestParam String mobile){
        String smsCode = shopUserService.sendSmsCodeForOldMobile(mobile);
        return Rets.success(smsCode);
    }
    @RequestMapping(value = "getWxOpenId",method = RequestMethod.POST)
    public  Object getWxOpenId(String code, HttpServletRequest request) {
        ShopUser user = shopUserService.getCurrentUser();
        boolean wxAuth = weixinService.isAuth(user,code);
        return wxAuth? Rets.success():Rets.failure("获取openid失败");
    }
    @RequestMapping(value = "getWxSign", method = RequestMethod.POST)
    public Object getWxSign(@RequestParam("url") String url) {
        Map<String, String> map = weixinService.getSign(url);
        return Rets.success(map);
    }
    @RequestMapping(value="uploadAvatar",method = RequestMethod.POST)
    public Object uploadAvatar(@RequestPart("file") MultipartFile multipartFile) {

        try {
            FileInfo fileInfo = fileService.upload(null,multipartFile);
            ShopUser user = shopUserService.getCurrentUser();
            user.setAvatar(String.valueOf(fileInfo.getRealFileName()));
            shopUserService.update(user);
            return Rets.success(fileInfo);
        } catch (Exception e) {
            logger.error("上传头像失败",e);
            return Rets.failure("上传头像失败");
        }
    }

    //更新用户登录的商圈id
    @RequestMapping(value = "/setTenantId",method = RequestMethod.GET)
    public Object setTenantId(Long tenantId){
        String token = getToken();
        Long idUser = JwtUtil.getUserId(token);
        int a = shopUserService.updateTenantId(idUser,tenantId);
        return Rets.success(a);
    }
    // 生成会员二维码
    @RequestMapping(value = "/getTradeScan",method = RequestMethod.GET)
    public Object getTradeScan(String id) throws ServiceException {
        String png_base64 = null;
        //返回二维码 并且构造支付包web支付对象
        ByteArrayOutputStream baOp = new ByteArrayOutputStream();
        try {
            BufferedImage bufImg = CyQrCodeUtil.createImage(id,235,235);
            // 生成二维码QRCode图片
            ImageIO.write(bufImg, "png", baOp);
            byte[] bytes = baOp.toByteArray();//转换成字节
            /**
             * jdk 17 替代方案
             */
            png_base64 = Base64Encoder.encode(bytes).trim();
            png_base64 = png_base64.replaceAll("\n", "").replaceAll("\r", "");//删除 \r\n
        } catch (Exception e) {
            throw new ServiceException("系统错误");
        }
       return Rets.success(png_base64);
    }
}
