package com.emall.flash.mobile.controller;

import com.emall.flash.bean.constant.CfgKey;
import com.emall.flash.bean.entity.shop.ShopUser;
import com.emall.flash.bean.entity.system.Cfg;
import com.emall.flash.bean.vo.JwtUser;
import com.emall.flash.bean.vo.UserInfo;
import com.emall.flash.bean.vo.front.Rets;
import com.emall.flash.bean.vo.shop.WechatInfo;
import com.emall.flash.cache.CacheDao;
import com.emall.flash.security.UserService;
import com.emall.flash.service.api.WeixinService;
import com.emall.flash.service.shop.ShopUserService;
import com.emall.flash.service.shop.TenantService;
import com.emall.flash.service.system.CfgService;
import com.emall.flash.utils.Maps;
import com.emall.flash.utils.RandomUtil;
import com.emall.flash.utils.StringUtil;
import com.emall.flash.web.controller.BaseController;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.Map;

/**
 * @author ：enilu
 * @date ：Created in 2020/6/6 20:32
 */
@RestController
@RequestMapping("/wechat")
public class WeChatController extends BaseController {
    @Autowired
    private ShopUserService shopUserService;
    @Autowired
    private UserService userService;
    @Autowired
    private WeixinService weixinService;
    @Autowired
    private CacheDao cacheDao;
    @Autowired
    private CfgService cfgService;
    @Autowired
    private TenantService tenantService;

    @RequestMapping(value = "getWxOpenId", method = RequestMethod.POST)
    public Object getWxOpenId(String code, HttpServletRequest request) {
        WechatInfo wechatInfo = weixinService.getWechatInfoByCode(code);
        if (wechatInfo == null) {
            return Rets.failure("获取微信消息失败");
        }
        return Rets.success(wechatInfo);
    }

    @RequestMapping(value = "getUserInfoByOpenId", method = RequestMethod.POST)
    public Object getUserInfoByOpenId(String code, String openId, HttpServletRequest request) {
        // 通过openId查询的用户
        ShopUser currentWechatUser = shopUserService.findByWechatOpenId(openId);
        // 获取当前登录用户
        ShopUser loginUser = shopUserService.getCurrentUser();
        Map result = null;
        if (currentWechatUser != null) {
            if (loginUser != null) {
                if (StringUtil.isNotEmpty(loginUser.getWechatOpenId())
                        && !loginUser.getWechatOpenId().equals(openId)) {
                    return Rets.failure("当前微信用户已绑定其他手机号");
                } else {
                    //当前用户通过手机登陆后获取微信信息
                    loginUser.setWechatOpenId(openId);
                   /* loginUser.setWechatHeadImgUrl(wechatInfo.getHeadUrl());
                    loginUser.setNickName(wechatInfo.getNickName());*/
                    shopUserService.update(loginUser);
                    result = login(loginUser);
                }
            } else {
                //当前用户通过微信自动登录
                result = login(currentWechatUser);
            }
        } else {
            if (loginUser == null) {
                String token = "";
                //用户手册通过微信自动登录+注册
                // 判断是否存在token
                Cfg tokenCfg = cfgService.getByCfgName(CfgKey.WX_ACCESS_TOKEN);
                if (tokenCfg.getCfgValue() == null) {
                    tokenCfg = new Cfg();
                    tokenCfg.setCfgName(CfgKey.WX_ACCESS_TOKEN);
                    tokenCfg.setCfgDesc("微信token，通过定时任务获取");
                    String accessToken = weixinService.getAccessToken();
                    tokenCfg.setCfgValue(accessToken);
                    cfgService.saveOrUpdate(tokenCfg);
                }
                token = tokenCfg.getCfgValue();
                // 获取手机号
                //初始化6位密码
                String initPassword = RandomUtil.getRandomString(6);
                loginUser = shopUserService.register(weixinService
                                .getPhoneNumberByAccessToken(token, code),
                        initPassword, openId);
            }
            result = login(loginUser);
            result.put("initPassword", loginUser.getPassword());
        }
        return Rets.success(result);
    }

    private Map login(ShopUser shopUser){
        String token = userService.loginForToken(new JwtUser(shopUser));
        shopUser.setLastLoginTime(new Date());
        shopUserService.update(shopUser);
        UserInfo userInfo = new UserInfo();
        BeanUtils.copyProperties(shopUser, userInfo);
        if (userInfo.getTenantId() != null)
            userInfo.setTenantName(tenantService.get(userInfo.getTenantId()).getName());
        userInfo.setRefreshWechatInfo(false);
        Map result = Maps.newHashMap(
                "user", userInfo,
                "token", token
        );
        return result;
    }

    @RequestMapping(value = "getWxSign", method = RequestMethod.POST)
    public Object getWxSign(@RequestParam("url") String url) {
        Map<String, String> map = weixinService.getSign(url);
        return Rets.success(map);
    }
}
