package com.emall.flash.service;

import com.emall.flash.bean.entity.shop.Order;
import com.emall.flash.bean.entity.shop.ShopUser;
import com.emall.flash.bean.enumeration.shop.OrderEnum;
import com.emall.flash.service.shop.OrderService;
import com.emall.flash.utils.HttpUtil;
import com.github.binarywang.wxpay.bean.notify.WxPayNotifyResponse;
import com.github.binarywang.wxpay.bean.notify.WxPayOrderNotifyResult;
import com.github.binarywang.wxpay.bean.notify.WxPayRefundNotifyResult;
import com.github.binarywang.wxpay.bean.order.WxPayMpOrderResult;
import com.github.binarywang.wxpay.bean.request.WxPayRefundRequest;
import com.github.binarywang.wxpay.bean.request.WxPayUnifiedOrderRequest;
import com.github.binarywang.wxpay.bean.result.WxPayRefundResult;
import com.github.binarywang.wxpay.constant.WxPayConstants;
import com.github.binarywang.wxpay.exception.WxPayException;
import com.github.binarywang.wxpay.service.WxPayService;
import org.apache.commons.io.IOUtils;
import org.nutz.json.Json;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 微信支付
 * @author ：enilu
 * @date ：Created in 2020/3/17 20:26
 */
@Service
public class WeixinPayService {
    @Autowired
    private WxPayService wxPayService;
    @Autowired
    private OrderService orderService;
    @Value("${wx.refund-notify-url}")
    private String refundNotifyUrl;
    @Value("${wx.mch-key}")
    private String key;

    private Logger logger = LoggerFactory.getLogger(WeixinPayService.class);
    public WxPayMpOrderResult prepare(ShopUser user, Order order){

        WxPayUnifiedOrderRequest orderRequest = new WxPayUnifiedOrderRequest();
        orderRequest.setOutTradeNo(order.getOrderSn());
        orderRequest.setOpenid(user.getWechatOpenId());
        orderRequest.setBody("订单：" + order.getOrderSn());
        BigDecimal totalPrice = order.getTotalPrice();
        orderRequest.setTotalFee(totalPrice.intValue());
        orderRequest.setSpbillCreateIp(HttpUtil.getIp());

        try {
            WxPayMpOrderResult result = wxPayService.createOrder(orderRequest);
            orderService.startPay(order);
            return  result;
        } catch (WxPayException e) {
            logger.error("微信支付异常",e);

        }
        return null;
    }

    public String resultNotify() {
        String xmlResult = null;

        try {
            HttpServletRequest request = HttpUtil.getRequest();
            xmlResult = IOUtils.toString(request.getInputStream(), request.getCharacterEncoding());

        } catch (IOException e) {
            logger.error("解析微信支付结果通知异常",e);
            return WxPayNotifyResponse.fail(e.getMessage());
        }

        WxPayOrderNotifyResult result = null;
        try {
            result = wxPayService.parseOrderNotifyResult(xmlResult);

            if(!WxPayConstants.ResultCode.SUCCESS.equals(result.getResultCode())){
                logger.error(xmlResult);
                throw new WxPayException("微信通知支付失败！");
            }
            if(!WxPayConstants.ResultCode.SUCCESS.equals(result.getReturnCode())){
                logger.error(xmlResult);
                throw new WxPayException("微信通知支付失败！");
            }
        } catch (WxPayException e) {
            e.printStackTrace();
            return WxPayNotifyResponse.fail(e.getMessage());
        }

        logger.info("处理腾讯支付平台的订单支付", Json.toJson(result));

        String orderSn = result.getOutTradeNo();
        String payId = result.getTransactionId();

        Integer totalFee =  result.getTotalFee();
        Order order = orderService.getByOrderSn(orderSn);
        if (order == null) {
            return WxPayNotifyResponse.fail("订单不存在 sn=" + orderSn);
        }

        // 检查这个订单是否已经处理过
        if (order.hasPayed()) {
            return WxPayNotifyResponse.success("订单已经处理成功!");
        }

        // 检查支付订单金额
        if (totalFee.intValue()!=order.getTotalPrice().intValue()) {
            return WxPayNotifyResponse.fail(order.getOrderSn() + " : 支付金额不符合 totalFee=" + totalFee);
        }

        order.setPayId(payId);
        orderService.paySuccess(order, OrderEnum.PayTypeEnum.UN_SEND.getKey());
        //todo 发送短信通知
        //todo 发送微信模板消息
        return WxPayNotifyResponse.success("支付成功！");
    }



    /**
     * 退款
     *
     * @return
     */
    public WxPayRefundResult refund( Order order) {
        WxPayRefundRequest wxPayRefundRequest = new WxPayRefundRequest();
        wxPayRefundRequest.setTransactionId(order.getPayId());
        wxPayRefundRequest.setOutRefundNo(String.valueOf(System.currentTimeMillis()));
        BigDecimal totalPrice = order.getTotalPrice();
        wxPayRefundRequest.setTotalFee(totalPrice.intValue());
        wxPayRefundRequest.setRefundFee(totalPrice.intValue());
        wxPayRefundRequest.setNotifyUrl(refundNotifyUrl);
        try {
            WxPayRefundResult refund = wxPayService.refund(wxPayRefundRequest);
            if (refund.getReturnCode().equals("SUCCESS") && refund.getResultCode().equals("SUCCESS")) {
                order.setStatus(OrderEnum.OrderStatusEnum.REFUND_ING.getId());
                order.setRefundTime(new Date());
                orderService.updateOrder(order);
                return refund;
            }

        } catch (WxPayException e) {
            logger.error("微信退款异常",e);
        }
        return null;
    }

    //退款回调
    public String refundNotify() {
        String xmlResult = null;

        try {
            HttpServletRequest request = HttpUtil.getRequest();
            xmlResult = IOUtils.toString(request.getInputStream(), request.getCharacterEncoding());

        } catch (IOException e) {
            logger.error("解析微信退款结果通知异常",e);
            return WxPayNotifyResponse.fail(e.getMessage());
        }
        logger.error(xmlResult);
        WxPayRefundNotifyResult result = null;
        try {
            result = wxPayService.parseRefundNotifyResult(xmlResult);
            if (!WxPayConstants.ResultCode.SUCCESS.equals(result.getReturnCode())) {
                logger.error(xmlResult);
                throw new WxPayException("微信退款-通知失败！");
            }
            if (!WxPayConstants.RefundStatus.SUCCESS.equals(result.getReqInfo().getRefundStatus())) {
                logger.error(xmlResult);
                throw new WxPayException("微信退款-通知失败");
            }
        } catch (WxPayException e) {
            logger.error("微信退款-通知失败", e);
            return WxPayNotifyResponse.fail(e.getMessage());
        }

        WxPayRefundNotifyResult.ReqInfo reqInfo = result.getReqInfo();

        logger.info("处理腾讯支付平台的订单退款", Json.toJson(result));

        String orderSn = reqInfo.getOutTradeNo();
//        String payId = reqInfo.getTransactionId();

        Integer totalFee =  reqInfo.getTotalFee();
        Order order = orderService.getByOrderSn(orderSn);
        if (order == null) {
            return WxPayNotifyResponse.fail("订单不存在 sn=" + orderSn);
        }

        // 检查这个订单是否已经处理过
        if (order.hasRefunded()) {
            return WxPayNotifyResponse.success("订单已经处理成功!");
        }

        // 检查支付订单金额
        if (totalFee.intValue()!=order.getTotalPrice().intValue()) {
            return WxPayNotifyResponse.fail(order.getOrderSn() + " : 支付金额不符合 totalFee=" + totalFee);
        }

        order.setStatus(OrderEnum.OrderStatusEnum.REFUND.getId());
        orderService.updateOrder(order);
        orderService.saveOrderLog(order, "退款成功");
        //todo 发送短信通知
        //todo 发送微信模板消息
        return WxPayNotifyResponse.success("退款成功！");
    }
}
